import matplotlib.pyplot as lab
import mpl_tri.tricontourset

def tricontour(*args, **kwargs):
    """
    Draw contour lines on a triangular grid.

    Call signatures:
      tricontour(x, y, triangles, z, **kwargs)
      tricontour(x, y, triangles, z, n, **kwargs)
      tricontour(x, y, triangles, z, levels, **kwargs)

    Arguments:
      x, y - arrays of point x and y-coordinates with shape (npoints).
      triangles - array of triangle point indices with shape (ntriangles,3).
      z - array of point z-values with shape (npoints).
      n - number of contour levels.
      levels - sequence of contour levels.

    Keyword arguments:
      mask - optional boolean mask array with shape (ntriangles).

      Other keyword arguments are passed on to the underlying matplotlib
      contour function.
    """
    kwargs['filled'] = False
    return mpl_tri.tricontourset.TriContourSet(lab.gca(), *args, **kwargs)

def tricontourf(*args, **kwargs):
    """
    Draw filled contours on a triangular grid.

    Call signatures:
      tricontourf(x, y, triangles, z, **kwargs)
      tricontourf(x, y, triangles, z, n, **kwargs)
      tricontourf(x, y, triangles, z, levels, **kwargs)

    Arguments:
      x, y - arrays of point x and y-coordinates with shape (npoints).
      triangles - array of triangle point indices with shape (ntriangles,3).
      z - array of point z-values with shape (npoints).
      n - number of contour levels (must be at least 2).
      levels - sequence of contour levels (must be at least 2).

    Keyword arguments:
      mask - optional boolean mask array with shape (ntriangles).

      Other keyword arguments are passed on to the underlying matplotlib
      contourf function.
    """
    kwargs['filled'] = True
    return mpl_tri.tricontourset.TriContourSet(lab.gca(), *args, **kwargs)
