import matplotlib.pyplot as lab
from matplotlib.collections import PolyCollection
import numpy as np

def trifill(x, y, triangles, **kwargs):
    """
    Draw triangular grid as filled triangles with or without edges.

    Arguments:
      x, y - arrays of point x and y-coordinates with shape (npoints).
      triangles - array of triangle point indices with shape (ntriangles,3).

    Keyword arguments:
      mask - optional boolean mask array with shape (ntriangles).

      Other keyword arguments are passed to underlying matplotlib
      PolyCollection, e.g. 'facecolors' and 'edgecolors'.
    """
    x = np.asarray(x)
    y = np.asarray(y)
    if x.shape != y.shape or len(x.shape) != 1:
        raise ValueError('x and y must be equal-length 1D arrays')
    
    triangles = np.asarray(triangles)
    if len(triangles.shape) != 2 or triangles.shape[1] != 3:
        raise ValueError('triangles must be a (?,3) array')
    if triangles.max() >= len(x):
        raise ValueError('triangles max element is out of bounds')
    if triangles.min() < 0:
        raise ValueError('triangles min element is out of bounds')

    # Optional mask.
    mask = kwargs.get('mask', None)
    if mask != None:
        kwargs.pop('mask')
	mask = np.asarray(mask, dtype=np.bool)
	if len(mask.shape) != 1 or mask.shape[0] != triangles.shape[0]:
	    raise ValueError('mask and triangles arrays must have same length')

    # Apply triangle mask.
    if mask != None:
        triangles = triangles.compress(1-mask, axis=0)

    # Plot.
    ax = lab.gca()
    vertices = np.concatenate((x[triangles][...,np.newaxis],
                               y[triangles][...,np.newaxis]), axis=2)
    collection = PolyCollection(vertices, **kwargs)
    ax.add_collection(collection)
    ax.autoscale_view()
