import matplotlib.pyplot as lab
from matplotlib.collections import PolyCollection
import numpy as np

def tripcolor(x, y, triangles, c, **kwargs):
    """
    Draw pseudocolor plot on triangular grid.

    Arguments:
      x, y - arrays of point x and y-coordinates with shape (npoints).
      triangles - array of triangle point indices with shape (ntriangles,3).
      c - array of point color values with shape (npoints).

    Keyword arguments:
      mask - optional boolean mask array with shape (ntriangles).

      Other keyword arguments are passed to underlying matplotlib
      PolyCollection, e.g. 'linewidths'.

    Limitations:
      Not complete...
    """
    x = np.asarray(x)
    y = np.asarray(y)
    if x.shape != y.shape or len(x.shape) != 1:
        raise ValueError('x and y must be equal-length 1D arrays')
	
    triangles = np.asarray(triangles)
    if len(triangles.shape) != 2 or triangles.shape[1] != 3:
        raise ValueError('triangles must be a (?,3) array')
    if triangles.max() >= len(x):
        raise ValueError('triangles max element is out of bounds')
    if triangles.min() < 0:
        raise ValueError('triangles min element is out of bounds')

    c = np.asarray(c)
    if c.shape != x.shape or len(c.shape) != 1:
        raise ValueError('c must be same length 1D array as x and y')

    # Optional mask.
    mask = kwargs.get('mask', None)
    if mask != None:
        kwargs.pop('mask')
        mask = np.asarray(mask, dtype=np.bool)
        if len(mask.shape) != 1 or mask.shape[0] != triangles.shape[0]:
            raise ValueError('mask and triangles arrays must have same length')

    # Apply triangle mask.
    if mask != None:
        triangles = triangles.compress(1-mask, axis=0)

    # Plot.
    ax = lab.gca()
    vertices = np.concatenate((x[triangles][...,np.newaxis],
                               y[triangles][...,np.newaxis]), axis=2)
    collection = PolyCollection(vertices, **kwargs)
    collection.set_array(c[triangles[:,0]])
    ax.add_collection(collection)
    ax.autoscale_view()
    
    return collection
