#ifndef BOUNDING_BOX_H
#define BOUNDING_BOX_H

#include "XY.h"

/**
 * 2D bounding box, which may be empty.
 */
class BoundingBox
{
public:
  BoundingBox()
    : empty(true)
  {}

  void add(const XY& point)
  {
    if (empty)
    {
      empty = false;
      lower = upper = point;
    }
    else
    {
      if      (point.x < lower.x) lower.x = point.x;
      else if (point.x > upper.x) upper.x = point.x;

      if      (point.y < lower.y) lower.y = point.y;
      else if (point.y > upper.y) upper.y = point.y;
    }
  }

  bool containsX(const double& x) const
  {
    return !empty && x >= lower.x && x <= upper.x;
  }

  friend ostream& operator<<(ostream& os, const BoundingBox& box)
  {
    if (box.empty)
    {
      return os << "<empty>";
    }
    else
    {
      return os << box.lower << " -> " << box.upper;
    }
  }

  // Consider these read-only.
  bool empty;
  XY lower, upper;
};

#endif // BOUNDING_BOX_H
