#ifndef HOLE_REMOVER_H
#define HOLE_REMOVER_H

#include "BoundingBox.h"
#include "Contour.h"

/**
 * Remover of holes in contours so that they are simply connected.
 */
class HoleRemover
{
public:
  HoleRemover();
 
  ~HoleRemover();

  void removeHoles(Contour& contour);

private:
  /**
   * Insert source hole ContourLine into target container ContourLine.
   */
  int insertHoleIntoContainer(ContourLine& target,
			      int targetEdge,
			      const ContourLine& source,
			      int sourceEdge,
			      const XY& point);

  /**
   * Test where the line connecting start and end points intersects the line
   * defined by x = point.x and return true if y <= point.y
   */
  bool intersectsBelowPoint(const XY& point,
			    const XY& start,
			    const XY& end,
			    double& y) const;



  struct HoleItem
  {
    HoleItem(int targetLine_,
	     int targetEdge_,
	     const XY& targetPoint_)
      : targetLine(targetLine_),
        targetEdge(targetEdge_),
        targetPoint(targetPoint_)
    {}

    bool operator<(const HoleItem& other) const
    {
      if (targetLine != other.targetLine)
      {
	return targetLine < other.targetLine;
      }
      else if (targetEdge != other.targetEdge)
      {
	return targetEdge > other.targetEdge;
      }
      else
      {
	return targetPoint.x > other.targetPoint.x;
      }
    }
    
    int targetLine, targetEdge;
    XY targetPoint;
  };

  struct LineItem
  {
    LineItem()
      : lowestEdge(-1), containingHoleCount(0), isHole(false)
    {}

    int lowestEdge, containingHoleCount;
    XY lowestPoint;
    BoundingBox bBox;
    bool isHole;
  };
};

#endif // HOLE_REMOVER_H

