#ifndef XY_H
#define XY_H

#include <cmath>
#include <iostream>
using namespace std;

struct XY
{
  XY()
  {}

  XY(const double& x_, const double& y_)
    : x(x_), y(y_)
  {}

  double angle() const
  {
    return atan2(y, x);
  }

  /**
   * Returns the z-component of the cross product of this and another XY.
   */
  double crossZ(const XY& other) const
  {
    return x*other.y - y*other.x;
  }

  friend ostream& operator<<(ostream& os, const XY& xy)
  {
    return os << '(' << xy.x << ' ' << xy.y << ')';
  }

  bool operator==(const XY& other) const
  {
    return x == other.x && y == other.y;
  }

  bool operator!=(const XY& other) const
  {
    return x != other.x || y != other.y;
  }

  bool operator<(const XY& other) const
  {
    if (y == other.y)
    {
      return x < other.x;
    }
    else
    {
      return y < other.y;
    }
  }

  XY operator-(const XY& other) const
  {
    return XY(x - other.x, y - other.y);
  }

  double x, y;
};

#endif // XY_H
